
#pragma once

#include "bit.hpp"
#include "error.hpp"
#include "math.hpp"
#include "types.hpp"
#include "writer.hpp"

namespace ox {

template<Integer_c Integer>
constexpr ox::Error writeItoa(Integer v, ox::Writer_c auto &writer) noexcept {
		if (v) {
			ox::ResizedInt_t<Integer, 64> mod = 1000000000000000000;
			ox::ResizedInt_t<Integer, 64> val = v;
			constexpr auto base = 10;
			auto it = 0;
			if (val < 0) {
				oxReturnError(writer.put('-'));
				++it;
			}
			while (mod) {
				auto digit = val / mod;
				val %= mod;
				mod /= base;
				if (it || digit) {
					ox::ResizedInt_t<Integer, 64> start = '0';
					if (digit >= 10) {
						start = 'a';
						digit -= 10;
					}
					oxReturnError(writer.put(static_cast<char>(start + digit)));
					++it;
				}
			}
		} else {
			// 0 is a special case
			oxReturnError(writer.put('0'));
		}
		return {};
}

constexpr ox::Error writeF32toa(float const v, ox::Writer_c auto &writer) noexcept {
	auto const raw = std::bit_cast<uint32_t>(v);
	if (raw >> 31) {
		oxReturnError(writer.put('-'));
	}
	auto const mantissa = raw & onMask<uint32_t>(23);
	auto const exp = ((raw >> 23) & 0xff) - 126;
	auto const p = ox::pow(2u, exp);
	auto const out = p * mantissa;
	oxReturnError(writeItoa(out, writer));
	return {};
}

}

#include "istring.hpp"
